@@ -33,7 +33,11 @@ class Server(object):
                          server to respond before disconnecting. The default
                          is 60 seconds.
     :param ping_interval: The interval in seconds at which the client pings
-                          the server. The default is 25 seconds.
+                          the server. The default is 25 seconds. For advanced
+                          control, a two element tuple can be given, where
+                          the first number is the ping interval and the second
+                          is a grace period added by the server. The default
+                          grace period is 5 seconds.
     :param max_http_buffer_size: The maximum size of a message when using the
                                  polling transport. The default is 100,000,000
                                  bytes.
@@ -83,7 +87,12 @@ class Server(object):
                  cors_credentials=True, logger=False, json=None,
                  async_handlers=True, monitor_clients=None, **kwargs):
         self.ping_timeout = ping_timeout
-        self.ping_interval = ping_interval
+        if isinstance(ping_interval, tuple):
+            self.ping_interval = ping_interval[0]
+            self.ping_interval_grace_period = ping_interval[1]
+        else:
+            self.ping_interval = ping_interval
+            self.ping_interval_grace_period = 5
         self.max_http_buffer_size = max_http_buffer_size
         self.allow_upgrades = allow_upgrades
         self.http_compression = http_compression
@@ -280,7 +289,8 @@ class Server(object):
                 pass
             else:
                 socket.close()
-                del self.sockets[sid]
+                if sid in self.sockets:  # pragma: no cover
+                    del self.sockets[sid]
         else:
             for client in six.itervalues(self.sockets):
                 client.close()
@@ -588,13 +598,19 @@ class Server(object):
                 'response': b'Unauthorized'}
 
     def _cors_allowed_origins(self, environ):
-        default_origin = None
+        default_origins = []
         if 'wsgi.url_scheme' in environ and 'HTTP_HOST' in environ:
-            default_origin = '{scheme}://{host}'.format(
-                scheme=environ['wsgi.url_scheme'], host=environ['HTTP_HOST'])
+            default_origins.append('{scheme}://{host}'.format(
+                scheme=environ['wsgi.url_scheme'], host=environ['HTTP_HOST']))
+            if 'HTTP_X_FORWARDED_HOST' in environ:
+                scheme = environ.get(
+                    'HTTP_X_FORWARDED_PROTO',
+                    environ['wsgi.url_scheme']).split(',')[0].strip()
+                default_origins.append('{scheme}://{host}'.format(
+                    scheme=scheme, host=environ['HTTP_X_FORWARDED_HOST'].split(
+                        ',')[0].strip()))
         if self.cors_allowed_origins is None:
-            allowed_origins = [default_origin] \
-                if default_origin is not None else []
+            allowed_origins = default_origins
         elif self.cors_allowed_origins == '*':
             allowed_origins = None
         elif isinstance(self.cors_allowed_origins, six.string_types):
@@ -643,7 +659,7 @@ class Server(object):
                 continue
 
             # go through the entire client list in a ping interval cycle
-            sleep_interval = self.ping_timeout / len(self.sockets)
+            sleep_interval = float(self.ping_timeout) / len(self.sockets)
 
             try:
                 # iterate over the current clients
